@@ -1,6 +1,4 @@
 /*
- *  xnec2c - GTK2-based version of nec2c, the C translation of NEC2
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,10 +12,13 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  The official website and doumentation for xnec2c is available here:
+ *    https://www.xnec2c.org/
  */
 
-#ifndef	EDITORS_H
-#define	EDITORS_H		1
+#ifndef EDITORS_H
+#define EDITORS_H       1
 
 #include "common.h"
 
@@ -26,16 +27,16 @@
 #define GEOM_NUM_FCOLS 7
 #define GEOM_NUM_ICOLS 2
 
-#define NEXT		TRUE
-#define PREVIOUS	FALSE
+#define NEXT        TRUE
+#define PREVIOUS    FALSE
 
 /* Editor actions that imply saving */
 enum SAVE_ACTION
 {
-  EDITOR_SAVE = 64,
-  EDITOR_NEW,
+  EDITOR_NEW = 64,
   EDITOR_EDIT,
-  EDITOR_APPLY
+  EDITOR_APPLY,
+  EDITOR_OK
 };
 
 enum EDITOR_FUNC
@@ -57,11 +58,14 @@ enum EDITOR_FUNC
 
   /* Helix editor function actions */
   HELIX_EDITOR_NTURN,
-  HELIX_EDITOR_LH,
-  HELIX_EDITOR_RH,
-  HELIX_EDITOR_LINKALL,
-  HELIX_EDITOR_LINKZO,
-  HELIX_EDITOR_LINKZHL,
+  HELIX_EDITOR_TSPACE,
+  HELIX_EDITOR_RH_HELIX,
+  HELIX_EDITOR_LH_HELIX,
+  HELIX_EDITOR_RH_SPIRAL,
+  HELIX_EDITOR_LH_SPIRAL,
+  HELIX_EDITOR_LINK_A1B1,
+  HELIX_EDITOR_LINK_B1A2,
+  HELIX_EDITOR_LINK_A2B2,
 
   /* Patch editor actions */
   PATCH_EDITOR_ARBT,
