@@ -20,193 +20,165 @@
 #include "gnuplot.h"
 #include "shared.h"
 
+// Touchstone save types:
+enum {
+	TOUCHSTONE_S1P,
+	TOUCHSTONE_S2P_MAXGAIN,
+	TOUCHSTONE_S2P_VIEWERGAIN,
+};
+
 /*-----------------------------------------------------------------------*/
 
 /* Save_FreqPlots_Gnuplot_Data()
  *
  * Saves frequency plots data to a file for gnuplot
  */
-  void
-Save_FreqPlots_Gnuplot_Data( char *filename )
+void Save_FreqPlots_Gnuplot_Data(char *filename)
 {
-  /* Abort if plot data not available */
-  if( isFlagClear(FREQ_LOOP_DONE) )
-    return;
+	FILE *fp = NULL;
+	time_t rawtime;
+	struct tm *info;
+	char buffer[80];
 
-  /* Used to calculate net gain */
-  double Zr, Zo, Zi;
-  int idx;
+	// Abort if plot data not available 
+	if (isFlagClear(FREQ_LOOP_DONE))
+	{
+		Notice(_("Gnuplot Data"), "Cannot save data while frequency loop is running", GTK_BUTTONS_OK);
+		return;
+	}
 
-  /* Open gplot file, abort on error */
-  FILE *fp = NULL;
-  if( !Open_File(&fp, filename, "w") )
-    return;
+	// Open gplot file, abort on error 
+	if (!Open_File(&fp, filename, "w"))
+		return;
+	setlocale(LC_NUMERIC, "C");
 
-  /* Plot max gain vs frequency, if possible */
-  if( isFlagSet(PLOT_GMAX) && isFlagSet(ENABLE_RDPAT) )
-  {
-    int nth, nph, pol;
-    gboolean no_fbr;
+	time( &rawtime );
+	info = localtime( &rawtime );
+	strftime(buffer, sizeof(buffer)-1, "%c (%F %H:%M:%S)", info);
 
-    double
-      gmax,     /* Max gain buffer */
-      netgain,  /* Viewer direction net gain buffer */
-      gdir_phi, /* Direction in phi of gain */
-      fbratio;  /* Front to back ratio */
+	fprintf(fp, "# %s - %s\n", rc_config.input_file, buffer);
+	fprintf(fp, _("# Reference impedance Z0 = %.2f Ohm\n"), calc_data.zo);
+	fprintf(fp, "#\n");
+	fprintf(fp, _("# To plot in gnuplot you can modify this command.  Presently it plots MHz vs zreal and zimag:\n"));
+	fprintf(fp, _("#    plot for [i=2:3]  'filename.gplot' using 1:i with lines smooth bezier title columnhead(i);\n"));
+	fprintf(fp, "#\n");
 
-    /* Find max gain and direction, F/B ratio */
-    no_fbr = FALSE;
-    netgain = 0;
+	meas_write_header_enc(fp, "\t", "\"", "\"");
+	meas_write_data(fp, "\t");
 
-    /* Polarization type and impedance */
-    pol = calc_data.pol_type;
-    Zo = calc_data.zo;
+	fclose(fp);
 
-    /* Save data for all frequency steps that were used */
-    fprintf( fp, _("# Gain and F/B Ratio vs Frequency\n") );
-    idx = 0;
-    for( int step = 0; step <= calc_data.last_step; step++ )
-    {
-      double fbdir;
-      int fbidx, mgidx;
+	setlocale(LC_NUMERIC, orig_numeric_locale);
+}
 
-      /* Index to gtot buffer where max gain
-       * occurs for given polarization type */
-      mgidx = rad_pattern[idx].max_gain_idx[pol];
 
-      /* Max gain for given polarization type */
-      gmax = rad_pattern[idx].gtot[mgidx] + Polarization_Factor(pol, idx, mgidx);
+void Save_FreqPlots_Touchstone(char *filename, int type)
+{
+	FILE *fp = NULL;
+	time_t rawtime;
+	struct tm *info;
+	char buffer[80];
+	int idx;
 
-      /* Net gain if selected */
-      if( isFlagSet(PLOT_NETGAIN) )
-      {
-        Zr = impedance_data.zreal[idx];
-        Zi = impedance_data.zimag[idx];
-        netgain = gmax + 10 * log10( 4 * Zr * Zo / (pow(Zr + Zo, 2) + pow( Zi, 2)) );
-      }
+	measurement_t meas;
 
-      /* Radiation angle/phi where max gain occurs */
-      gdir_phi = rad_pattern[idx].max_gain_phi[pol];
+	// Abort if plot data not available
+	if (isFlagClear(FREQ_LOOP_DONE))
+	{
+		Notice(_("Touchstone Data"), "Cannot save data while frequency loop is running", GTK_BUTTONS_OK);
+		return;
+	}
 
-      /* Find F/B direction in theta */
-      fbdir = 180.0 - rad_pattern[idx].max_gain_tht[pol];
-      if( fpat.dth == 0.0 ) nth = 0;
-      else nth = (int)( fbdir/fpat.dth + 0.5 );
+	// Open gplot file, abort on error
+	if (!Open_File(&fp, filename, "w"))
+		return;
+	setlocale(LC_NUMERIC, "C");
 
-      /* If the antenna is modelled over ground, then use the same
-       * theta as the max gain direction, relying on phi alone to
-       * take us to the back. Patch supplied by Rik van Riel AB1KW
-       */
-      if( (nth >= fpat.nth) || (nth < 0) )
-      {
-        fbdir = rad_pattern[idx].max_gain_tht[pol];
-        if( fpat.dth == 0.0 ) nth = 0;
-        else nth = (int)( fbdir/fpat.dth + 0.5 );
-      }
+	time( &rawtime );
+	info = localtime( &rawtime );
+	strftime(buffer, sizeof(buffer)-1, "%c (%F %H:%M:%S)", info);
 
-      /* Find F/B direction in phi */
-      fbdir = gdir_phi + 180.0;
-      if( fbdir >= 360.0 ) fbdir -= 360.0;
-      nph = (int)( fbdir/fpat.dph + 0.5 );
+	fprintf(fp, "! %s - %s\n", rc_config.input_file, buffer);
+	fprintf(fp, _("! Reference impedance Z0 = %.2f Ohm\n"), calc_data.zo);
+	fprintf(fp, "!\n");
 
-      /* No F/B calc. possible if no phi step at +180 from max gain */
-      if( (nph >= fpat.nph) || (nph < 0) )
-        no_fbr = TRUE;
+	char *format = "{mhz}\t{s11_real}\t{s11_ang}\n";
+	switch (type)
+	{
+		case TOUCHSTONE_S1P:
+			fprintf(fp, "!MHz\tS11(Real)\tS11(Ang)\n");
+			format = "{mhz}\t{s11_real}\t{s11_ang}\n";
+			break;
 
-      /* Index to gtot buffer for gain in back direction */
-      fbidx = nth + nph*fpat.nth;
 
-      /* Front to back ratio */
-      fbratio  = pow( 10.0, gmax / 10.0 );
-      fbratio /= pow( 10.0, (rad_pattern[idx].gtot[fbidx] +
-            Polarization_Factor(pol, idx, fbidx)) / 10.0 );
-      fbratio = 10.0 * log10( fbratio );
+		// For .s2p files gain is used as S21 and S12.  We assume they
+		// are passive so S21==S12.  S22 is a bit of a mystery, so we 
+		// assume that all S22 behavior is normalized into S11 and thus
+		// S22 is deminimus and set it to -100 dB.
+		//
+		case TOUCHSTONE_S2P_MAXGAIN:
+			format = "{mhz}\t{s11_real}\t{s11_ang}\t{gain_max}\t0\t{gain_max}\t0\t-100\t0\n";
+			fprintf(fp, "!MHz\tS11(Real)\tS11(Ang)\tS21(Real)\tS21(Ang)\tS12(Real)\tS12(Ang)\tS22(Real)\tS22(Ang)\n");
+			break;
 
-      if( no_fbr && isFlagClear(PLOT_NETGAIN) ) /* Plot max gain only */
-        fprintf( fp, "%13.6E %10.3E\n", save.freq[idx], gmax );
-      else if( isFlagSet(PLOT_NETGAIN) ) /* Plot max gain and net gain */
-        fprintf( fp, "%13.6E %10.3E %10.3E\n", save.freq[idx], gmax, netgain );
-      else if( !no_fbr ) /* Plot max gain and F/B ratio */
-        fprintf( fp, "%13.6E %10.3E %10.3E\n", save.freq[idx], gmax, fbratio );
+		case TOUCHSTONE_S2P_VIEWERGAIN:
+			format = "{mhz}\t{s11_real}\t{s11_ang}\t{gain_viewer}\t0\t{gain_viewer}\t0\t-100\t0\n";
+			fprintf(fp, "!MHz\tS11(Real)\tS11(Ang)\tS21(Real)\tS21(Ang)\tS12(Real)\tS12(Ang)\tS22(Real)\tS22(Ang)\n");
+			break;
 
-      idx++;
-    } /* for( int step = 0; step <= calc_data.freq_loop_data[fr].last_step; step++ ) */
+		default:
+			BUG("This should never happen. touchstone type=%d\n", type);
+	}
 
-    /* Plot gain direction in phi and theta */
-    idx = 0;
-    if( isFlagSet(PLOT_GAIN_DIR) )
-    {
-      fprintf( fp, "\n\n" );
-      fprintf( fp, _("# Direction of gain in theta and phi\n") );
-      for( int step = 0; step <= calc_data.last_step; step++ )
-      {
-        double gdir_tht; /* Direction in theta of gain */
+	fprintf(fp, "# MHz S DB R %g\n", calc_data.zo);
 
-        /* Radiation angle/phi where max gain occurs */
-        gdir_tht = 90.0 - rad_pattern[idx].max_gain_tht[pol];
-        gdir_phi = rad_pattern[idx].max_gain_phi[pol];
-        fprintf( fp, "%13.6E %10.3E %10.3E\n", save.freq[idx], gdir_tht, gdir_phi );
+	for (idx = 0; idx < calc_data.steps_total; idx++)
+	{
+		meas_calc(&meas, idx);
+		meas_write_format(&meas, format, fp);
+	}
 
-        idx++;
-      } /* for( int step = 0; step <= calc_data.freq_loop_data[fr].last_step; step++ ) */
-    } /* if( isFlagSet(PLOT_GAIN_DIR) ) */
+	fclose(fp);
 
-    fprintf( fp, "\n\n" );
-  } /* if( isFlagSet(PLOT_GMAX) && isFlagSet(ENABLE_RDPAT) ) */
+	setlocale(LC_NUMERIC, orig_numeric_locale);
+}
 
-  /* Plot VSWR vs freq */
-  if( isFlagSet(PLOT_VSWR) )
-  {
-    double vswr, gamma;
-    double zrpro2, zrmro2, zimag2;
+void Save_FreqPlots_S1P(char *filename)
+{
+	Save_FreqPlots_Touchstone(filename, TOUCHSTONE_S1P);
+}
 
-    /* Calculate VSWR */
-    fprintf( fp, _("# VSWR vs Frequency\n") );
-    idx = 0;
-    for( int step = 0; step <= calc_data.last_step; step++ )
-    {
-      zrpro2 = impedance_data.zreal[idx] + calc_data.zo;
-      zrpro2 *= zrpro2;
-      zrmro2 = impedance_data.zreal[idx] - calc_data.zo;
-      zrmro2 *= zrmro2;
-      zimag2 = impedance_data.zimag[idx] * impedance_data.zimag[idx];
-      gamma = sqrt( (zrmro2 + zimag2)/(zrpro2 + zimag2) );
-      vswr = (1+gamma)/(1-gamma);
-      if( vswr > 10.0 ) vswr = 10.0;
-      fprintf( fp, "%13.6E %10.3E\n", save.freq[idx], vswr );
+void Save_FreqPlots_S2P_Max_Gain(char *filename)
+{
+	Save_FreqPlots_Touchstone(filename, TOUCHSTONE_S2P_MAXGAIN);
+}
 
-      idx++;
-    }
+void Save_FreqPlots_S2P_Viewer_Gain(char *filename)
+{
+	Save_FreqPlots_Touchstone(filename, TOUCHSTONE_S2P_VIEWERGAIN);
+}
 
-    fprintf( fp, "\n\n" );
-  } /* if( isFlagSet(PLOT_VSWR) ) */
+void Save_FreqPlots_CSV(char *filename)
+{
+	/* Open Optimizer csv file */
+	FILE *fp = NULL;
 
-  /* Plot z-real and z-imag */
-  if( isFlagSet(PLOT_ZREAL_ZIMAG) )
-  {
-    fprintf( fp, _("# Z real & Z imaginary vs Frequency\n") );
-    idx = 0;
-    for( int step = 0; step <= calc_data.last_step; step++ )
-      fprintf( fp, "%13.6E %10.3E %10.3E\n",
-          save.freq[idx], impedance_data.zreal[idx], impedance_data.zimag[idx] );
-    fprintf( fp, "\n\n" );
+	// Abort if plot data not available
+	if (isFlagClear(FREQ_LOOP_DONE))
+	{
+		Notice(_("Touchstone Data"), "Cannot save data while frequency loop is running", GTK_BUTTONS_OK);
+		return;
+	}
 
-    idx++;
-  } /* if( isFlagSet(PLOT_ZREAL_ZIMAG) ) */
+	if (!Open_File(&fp, filename, "w"))
+		return;
 
-  /* Plot z-magn and z-phase */
-  if( isFlagSet(PLOT_ZMAG_ZPHASE) )
-  {
-    idx = 0;
-    fprintf( fp, _("# Z magnitude & Z phase vs Frequency\n") );
-    for( int step = 0; step <= calc_data.last_step; step++ )
-      fprintf( fp, "%13.6E %10.3E %10.3E\n",
-          save.freq[idx], impedance_data.zmagn[idx], impedance_data.zphase[idx] );
-    idx++;
-  } /* if( isFlagSet(PLOT_ZREAL_ZIMAG) ) */
+	meas_write_header(fp, ",");
+	meas_write_data(fp, ",");
+
+	fclose(fp);
+}
 
-  fclose(fp);
-} /* Save_FreqPlots_Gnuplot_Data() */
 
 /*-----------------------------------------------------------------------*/
 
@@ -235,6 +207,8 @@ Save_RadPattern_Gnuplot_Data( char *filename )
     if( !Open_File(&fp, filename, "w") )
       return;
 
+    setlocale(LC_NUMERIC, "C");
+
     /* Reference for scale factor used in
      * normalizing field strength values */
     if( fpat.near ) /* Spherical co-ordinates */
@@ -389,6 +363,9 @@ Save_RadPattern_Gnuplot_Data( char *filename )
     /* Open gplot file, abort on error */
     if( !Open_File(&fp, filename, "w") )
       return;
+
+    setlocale(LC_NUMERIC, "C");
+
     fprintf( fp, _("# Radiation Pattern\n") );
 
     /* Distance of rdpattern point nearest to xyz origin */
@@ -429,9 +406,80 @@ Save_RadPattern_Gnuplot_Data( char *filename )
     } /* for( nph = 0; nph < fpat.nph; nph++ ) */
   } /* if( isFlagSet(ENABLE_RDPAT) && (calc_data.freq_step >= 0) ) */
 
+  setlocale(LC_NUMERIC, orig_numeric_locale);
+
   if( fp != NULL ) fclose(fp);
 } /* Save_RadPattern_Gnuplot_Data() */
 
+
+void Save_RadPattern_CSV(char *filename)
+{
+	FILE *fp = NULL;
+
+	if (!Open_File(&fp, filename, "w"))
+		return;
+
+	fprintf(fp, "mhz,phi,theta,gain_total,gain_horiz,gain_vert,gain_rhcp,gain_lhcp\n");
+
+	setlocale(LC_NUMERIC, "C");
+	// Distance of rdpattern point nearest to xyz origin
+	// Convert radiation pattern values to points in 3 d space in x, y, z axis
+
+	double theta, phi, r;
+
+    /* theta and phi step in rads */
+    double dth = (double)fpat.dth * (double)TORAD;
+    double dph = (double)fpat.dph * (double)TORAD;
+
+	int calc_idx, idx, nph, nth, pol;
+
+	if (isFlagSet(ENABLE_RDPAT) && (calc_data.freq_step >= 0))
+	{
+		for (calc_idx = 0; calc_idx < calc_data.steps_total; calc_idx++)
+		{
+			// Step phi angle
+			idx = 0;
+			phi = (double)fpat.phis * (double)TORAD; // In rads
+
+			for (nph = 0; nph < fpat.nph; nph++)
+			{
+				theta = (double) fpat.thets * (double) TORAD;	// In rads
+
+				// Step theta angle
+				for (nth = 0; nth < fpat.nth; nth++)
+				{
+					// Distance of pattern point from the xyz origin
+					r = rad_pattern[calc_idx].gtot[idx];
+
+					// mhz,phi,theta
+					fprintf(fp, "%.6f,%.17g,%.17g,",
+						save.freq[calc_idx],
+						phi * TODEG,
+						theta * TODEG);
+
+					// POL_TOTAL, POL_HORIZ, POL_VERT, POL_RHCP, POL_LHCP
+					for (pol = 0; pol < NUM_POL; pol++)
+						fprintf(fp, "%.17g%s", 
+							r + Polarization_Factor(pol, calc_idx, idx),
+							(pol == NUM_POL-1 ? "\n" : ",")
+							);
+
+					// Step theta in rads
+					theta += dth;
+					idx++;
+				} // for( nth = 0; nth < fpat.nth; nth++ )
+
+				// Step phi in rads
+				phi += dph;
+			} // for( nph = 0; nph < fpat.nph; nph++ )
+		}
+	} // if( isFlagSet(ENABLE_RDPAT) && (calc_data.freq_step >= 0) )
+
+	setlocale(LC_NUMERIC, orig_numeric_locale);
+
+	fclose(fp);
+}
+
 /*-----------------------------------------------------------------------*/
 
 /* Save_Struct_Gnuplot_Data()
@@ -446,6 +494,7 @@ Save_Struct_Gnuplot_Data( char *filename )
   /* Open gplot file, abort on error */
   if( !Open_File(&fp, filename, "w") )
     return;
+  setlocale(LC_NUMERIC, "C");
 
   /* Output if patch segs and no new input pending */
   if( data.m && isFlagClear(INPUT_PENDING) )
@@ -501,8 +550,62 @@ Save_Struct_Gnuplot_Data( char *filename )
     } /* for( idx = 1; idx < data.n; idx++ ) */
   } /* if( data.n && isFlagSet(INPUT_PENDING) ) */
 
+  setlocale(LC_NUMERIC, orig_numeric_locale);
   fclose( fp );
 } /* Save_Struct_Gnuplot_Data() */
 
 /*-----------------------------------------------------------------------*/
 
+void Save_Currents_CSV(char *filename)
+{
+	FILE *fp = NULL;
+
+	if (!crnt.valid)
+	{
+		Notice("Save Currents and Charges to CSV",
+			"You must enable current data by clicking \"Currents\" or \"Charges\""
+			"in the main xnec2c window.", GTK_BUTTONS_OK);
+		return;
+	}
+
+	if (!Open_File(&fp, filename, "w"))
+		return;
+
+	setlocale(LC_NUMERIC, "C");
+
+	fprintf(fp, "mhz,seg,tag,"
+		"current_real,current_imag,current_mag,"  // Current in Amperes
+		"charge_real,charge_imag,charge_mag,"     // Charge in Coulombs
+		"x1,y1,z1,x2,y2,z2\n");
+
+	int idx;
+	double wavelength = data.wlam;
+	double charge_scale = 1.0E-6 / calc_data.freq_mhz;
+
+	for (idx = 0; idx < data.n; idx++)
+	{
+		fprintf(fp, "%.6f,%d,%d,"
+			"%.17g,%.17g,%.17g,"  // Current (A)
+			"%.17g,%.17g,%.17g,"  // Charge (C)
+			"%.17g,%.17g,%.17g,%.17g,%.17g,%.17g\n",
+			calc_data.freq_mhz,
+			idx+1, data.itag[idx],
+
+			// Currents
+			creal(crnt.cur[idx]) * wavelength,
+			cimag(crnt.cur[idx]) * wavelength,
+			cabs(crnt.cur[idx]) * wavelength,
+
+			// Charges
+			crnt.bir[idx] * charge_scale,
+			crnt.bii[idx] * charge_scale,
+			cabs(cmplx(crnt.bir[idx], crnt.bii[idx])) * charge_scale,
+
+			// Segment endpoints
+			data.x1[idx], data.y1[idx], data.z1[idx],
+			data.x2[idx], data.y2[idx], data.z2[idx]);
+	}
+
+	setlocale(LC_NUMERIC, orig_numeric_locale);
+	fclose(fp);
+}
