@@ -18,6 +18,8 @@
  */
 
 #include <ctype.h>
+#include <sys/stat.h>
+#include "shared.h"
 #include "rc_config.h"
 #include "mathlib.h"
 
@@ -29,6 +31,8 @@
  */
 
 rc_config_vars_t rc_config_vars[] = {
+	// Historical typo "Vesrsion", but it cannot changed for backwards
+	// compatibility because we parse the comment:
 	{ .desc = "Application Vesrsion", .format = "%s", .ro = 1, 
 		.vars = { PACKAGE_STRING } },
 
@@ -63,7 +67,10 @@ rc_config_vars_t rc_config_vars[] = {
 		.vars = { &rc_config.main_left_hand } },
 
 	{ .desc = "Main Window Frequency loop start state", .format = "%d",
-		.vars = { &rc_config.main_loop_start } },
+		.vars = { &rc_config.main_loop_start },
+		.builder_window = &main_window_builder,
+		.builder_check_menu_item_id = "config_main_loop_start"
+		},
 
 	{ .desc = "Main Window Rotate spinbutton state", .format = "%d",
 		.vars = { &rc_config.main_rotate_spinbutton } },
@@ -150,7 +157,7 @@ rc_config_vars_t rc_config_vars[] = {
 		.vars = { &rc_config.mathlib_idx }, .init = mathlib_config_init },
 
 	{ .desc = "Selected Batch Mathlib", .format = "%d",
-		.vars = { &rc_config.mathlib_batch_idx } },
+		.vars = { &rc_config.mathlib_batch_idx }, .init = mathlib_config_init },
 
 	{ .desc = "Selected Mathlib Benchmarks",
 		.parse = mathlib_config_benchmark_parse,
@@ -170,12 +177,62 @@ rc_config_vars_t rc_config_vars[] = {
 
 	{ .desc = "Round X Axis", .format = "%d",
 		.vars = { &rc_config.freqplots_round_x_axis } },
+
+	{ .desc = "Optimizer Write CSV", .format = "%d", .batch_mode_skip = TRUE,
+		.builder_window = &main_window_builder, .builder_check_menu_item_id = "optimizer_write_csv",
+		.vars = { &rc_config.opt_write_csv } },
+
+	{ .desc = "Optimizer Write S1P", .format = "%d", .batch_mode_skip = TRUE,
+		.builder_window = &main_window_builder, .builder_check_menu_item_id = "optimizer_write_s1p",
+		.vars = { &rc_config.opt_write_s1p } },
+
+	{ .desc = "Optimizer Write S2P Max Gain", .format = "%d", .batch_mode_skip = TRUE,
+		.builder_window = &main_window_builder, .builder_check_menu_item_id = "optimizer_write_s2p_max_gain",
+		.vars = { &rc_config.opt_write_s2p_max_gain } },
+
+	{ .desc = "Optimizer Write S2P Viewer Gain", .format = "%d", .batch_mode_skip = TRUE,
+		.builder_window = &main_window_builder, .builder_check_menu_item_id = "optimizer_write_s2p_viewer_gain",
+		.vars = { &rc_config.opt_write_s2p_viewer_gain } },
+
+	{ .desc = "Optimizer Write CSV Radiation Pattern", .format = "%d", .batch_mode_skip = TRUE,
+		.builder_window = &main_window_builder, .builder_check_menu_item_id = "optimizer_write_rdpat",
+		.vars = { &rc_config.opt_write_rdpat } },
+
+	{ .desc = "Optimizer Write CSV Currents/Charges", .format = "%d", .batch_mode_skip = TRUE,
+		.builder_window = &main_window_builder, .builder_check_menu_item_id = "optimizer_write_currents",
+		.vars = { &rc_config.opt_write_currents} },
+
+	{ .desc = "Frequency Plots window is open", .format = "%d",
+		.vars = { &rc_config.freqplots_is_open } },
+
+	{ .desc = "Radiation Pattern Window window is open", .format = "%d",
+		.vars = { &rc_config.rdpattern_is_open } },
 };
 
 
 rc_config_vars_t rc_config_vars[];
 int num_rc_config_vars = sizeof(rc_config_vars) / sizeof(rc_config_vars_t);
 
+char *get_conf_dir(char *s, int len)
+{
+	char *home = getenv("HOME");
+	if (home == NULL || strlen(home) == 0)
+	{
+		if (getcwd(s, len) == NULL)
+		{
+			pr_warn("unable to getcwd(): %s\n", strerror(errno));
+			strncpy(s, "./", len);
+		}
+
+		pr_warn("environment variable HOME is undefined, using %s\n", s);
+	}
+	else
+	{
+		strncpy(s, home, len);
+	}
+
+	return s;
+}
 
 // Trim the newline
 void chomp(char *line)
@@ -204,10 +261,13 @@ rc_config_vars_t *find_var(char *s)
 	return NULL;
 }
 
-// Rease the line into the variable's .vars reference(s)
-int parse_var(rc_config_vars_t *v, char *line)
+// Read the line into the variable's .vars reference(s)
+int parse_var(rc_config_vars_t *v, char *line, char *locale)
 {
+	char *format = NULL;
 	int count = 0;
+	int check = 0;
+	size_t mreq;
 
 	// Use the parse function if available:
 	if (v->parse != NULL)
@@ -217,23 +277,41 @@ int parse_var(rc_config_vars_t *v, char *line)
 	if (v->ro || (v->vars[0] == NULL && v->vars[1] == NULL))
 		return 0;
 
+	mreq = strlen(v->format)+4;
+	mem_alloc((void**)&format, mreq, __LOCATION__);
+	snprintf(format, mreq-1, "%s%%n", v->format);
+
+	setlocale(LC_NUMERIC, locale);
+
 	if (strcmp(v->format, "%d") == 0)
-		count = sscanf(line, v->format, (int*)v->vars[0]);
+		count = sscanf(line, format, (int *)v->vars[0], &check);
 	else if (strcmp(v->format, "%s") == 0)
 	{
 		strncpy((char*)v->vars[0], line, v->size - 1);
 		count = 1;
+		check = strlen(line);
 	}
 	else if (strcmp(v->format, "%f") == 0)
-		count = sscanf(line, v->format, (double*)v->vars[0]);
+		count = sscanf(line, format, (double *)v->vars[0], &check);
 	else if (strcmp(v->format, "%lf") == 0)
-		count = sscanf(line, v->format, (double*)v->vars[0]);
+		count = sscanf(line, format, (double *)v->vars[0], &check);
 	else if (strcmp(v->format, "%d,%d") == 0)
-		count = sscanf(line, v->format, (int*)v->vars[0], (int*)v->vars[1]);
+		count = sscanf(line, format, (int *)v->vars[0], (int *)v->vars[1], &check);
 	else if (strcmp(v->format, "%f,%f") == 0)
-		count = sscanf(line, v->format, (float*)v->vars[0], (float*)v->vars[1]);
+		count = sscanf(line, format, (float *)v->vars[0], (float *)v->vars[1], &check);
 	else if (strcmp(v->format, "%lf,%lf") == 0)
-		count = sscanf(line, v->format, (double*)v->vars[0], (double*)v->vars[1]);
+		count = sscanf(line, format, (double *)v->vars[0], (double *)v->vars[1], &check);
+
+	setlocale(LC_NUMERIC, orig_numeric_locale);
+
+	free_ptr((void**)&format);
+
+	if (check != strlen(line))
+	{
+		pr_warn("%s (locale=%s): only matchd %d of %lu chars, trying another locale: %s\n",
+		        v->desc, locale, check, (unsigned long)strlen(line), line);
+		return 0;
+	}
 
 	// `count` contains the number of vars parsed, return true
 	// if the count matches the number of vars, otherwise it failed
@@ -255,6 +333,8 @@ int fprint_var(FILE *fp, rc_config_vars_t *v)
 	if (v->vars[0] == NULL && v->vars[1] == NULL)
 		return 0;
 
+	setlocale(LC_NUMERIC, "C");
+
 	if (strcmp(v->format, "%d") == 0)
 		count = fprintf(fp, v->format, *(int*)v->vars[0]);
 	else if (strcmp(v->format, "%s") == 0)
@@ -270,17 +350,56 @@ int fprint_var(FILE *fp, rc_config_vars_t *v)
 	else if (strcmp(v->format, "%lf,%lf") == 0)
 		count = fprintf(fp, v->format, *(double*)v->vars[0], *(double*)v->vars[1]);
 
+	setlocale(LC_NUMERIC, orig_numeric_locale);
+
 	return count;
 }
 
+
+/*
+ * rc_callback_check_menu_item()
+ *
+ * Use as a callback in resources/xnec2c.glade for GtkCheckMenuItem to handle
+ * the checkbox toggle and also set the referenced variable defined in
+ * rc_config_vars.
+ *
+ */
+void rc_callback_check_menu_item(GtkCheckMenuItem *menuitem, gpointer user_data)
+{
+	for (int i = 0; i < num_rc_config_vars; i++)
+	{
+		if (rc_config_vars[i].builder_check_menu_item_id == NULL && rc_config_vars[i].builder_window != NULL)
+			BUG("rc_config: %s: builder_check_menu_item_id is NULL but builder_window is defined.\n",
+				rc_config_vars[i].desc);
+		else if (rc_config_vars[i].builder_check_menu_item_id != NULL && rc_config_vars[i].builder_window == NULL)
+			BUG("rc_config: %s: builder_window is NULL but builder_check_menu_item_id is defined.\n",
+				rc_config_vars[i].desc);
+
+		if (rc_config_vars[i].builder_check_menu_item_id == NULL || rc_config_vars[i].builder_window == NULL)
+			continue;
+
+		GtkBuilder *bw = *rc_config_vars[i].builder_window;
+		GtkWidget *w = Builder_Get_Object(bw, rc_config_vars[i].builder_check_menu_item_id);
+		if (GTK_CHECK_MENU_ITEM(w) == menuitem)
+		{
+			pr_debug("match: %s\n", rc_config_vars[i].builder_check_menu_item_id);
+
+			// This is a toggle button, it will always be an int:
+			int *var = (int*)rc_config_vars[i].vars[0];
+			*var = gtk_check_menu_item_get_active(menuitem);
+		}
+	}
+}
+
+
 /*------------------------------------------------------------------------*/
 
 /* Get_Widget_Geometry()
  *
  * Gets a widget's size and position
+ * Return: 1 if open, 0 if closed.
  */
-  static void
-Get_Window_Geometry(
+int Get_Window_Geometry(
     GtkWidget *window,
     gint *x, gint *y, gint *width, gint *height )
 {
@@ -288,11 +407,14 @@ Get_Window_Geometry(
   {
     gtk_window_get_size( GTK_WINDOW(window), width, height );
     gtk_window_get_position( GTK_WINDOW(window), x, y );
+	pr_debug("window: x=%-5d y=%-5d w=%-5d h=%-5d  title: \"%s\"\n",
+		*x, *y, *width, *height,
+		gtk_window_get_title(GTK_WINDOW(window)));
+    return 1;
   }
   else /* Disable window restoring */
   {
-    *x = *y = 0;
-    *width = *height = 0;
+    return 0;
   }
 
 } /* Get_Widget_Geometry() */
@@ -310,33 +432,33 @@ Get_Window_Geometry(
 Create_Default_Config( void )
 {
   char
+    home[PATH_MAX],
     line[LINE_LEN],
     cfg_file[FILENAME_LEN];   /* Path to config file */
 
   FILE *fp = NULL;
 
   /* Setup file path to xnec2c config file */
-  snprintf( cfg_file, sizeof(cfg_file), "%s/%s", getenv("HOME"), CONFIG_FILE );
+  snprintf( cfg_file, sizeof(cfg_file), "%s/%s", get_conf_dir(home, sizeof(home)), CONFIG_FILE );
   fp = fopen( cfg_file, "r" );
   if( fp != NULL )
   {
     /* Read Application Version */
     if( Load_Line(line, fp) == EOF )
     {
-      fprintf( stderr, _("xnec2c: failed to read Application Version\n") );
-      Close_File( &fp );
-      return( FALSE );
+      pr_err("%s: EOF reading application version.\n", cfg_file);
+      strcpy(line, "(unknown)");
     }
 
     /* Produce fresh default config file if version number new */
     if( strncmp(line, PACKAGE_STRING, sizeof(line)) != 0 )
-      printf( _("xnec2c: warning: existing config file version differs: %s != %s\n"), line, PACKAGE_STRING );
+      pr_notice("existing config file version differs: %s != %s\n", line, PACKAGE_STRING);
 
     Close_File( &fp );
   } /* if( (fp = fopen(cfg_file, "r")) != NULL ) */
 
   /* For main window */
-  Strlcpy( rc_config.working_dir, getenv("HOME"), sizeof(rc_config.working_dir) );
+  Strlcpy( rc_config.working_dir, get_conf_dir(home, sizeof(home)), sizeof(rc_config.working_dir) );
   Strlcat( rc_config.working_dir, "/", sizeof(rc_config.working_dir) );
   rc_config.main_width  = 600;
   rc_config.main_height = 400;
@@ -354,12 +476,19 @@ Create_Default_Config( void )
   rc_config.main_incline_spinbutton = 45;
   rc_config.main_zoom_spinbutton    = 100;
 
+  // The *_is_open values below default to "1" for backward compatiblity.  It
+  // still does not open the window if width/height are undefined so defaults
+  // are consistent.  However, if they are defined but *_is_open is not defined
+  // (ie, <= v4.1.12) then the specified windows will open.  After the first
+  // run the new version will use xnec2c.conf for *_is_open values.
+
   /* For rdpattern window */
+  rc_config.rdpattern_is_open = 1;
   rc_config.rdpattern_width  = 0;
   rc_config.rdpattern_height = 0;
-  rc_config.rdpattern_x = 0;
-  rc_config.rdpattern_y = 0;
-  rc_config.rdpattern_gain_togglebutton = 0;
+  rc_config.rdpattern_x = -1;
+  rc_config.rdpattern_y = -1;
+  rc_config.rdpattern_gain_togglebutton = 1;
   rc_config.rdpattern_eh_togglebutton   = 0;
   rc_config.rdpattern_e_field = 1;
   rc_config.rdpattern_h_field = 1;
@@ -370,14 +499,15 @@ Create_Default_Config( void )
   rc_config.gain_style = GS_LINP;
 
   /* For freq plots window */
+  rc_config.freqplots_is_open = 1;
   rc_config.freqplots_width  = 0;
   rc_config.freqplots_height = 0;
-  rc_config.freqplots_x = 0;
-  rc_config.freqplots_y = 0;
-  rc_config.freqplots_gmax_togglebutton    = 0;
+  rc_config.freqplots_x = -1;
+  rc_config.freqplots_y = -1;
+  rc_config.freqplots_gmax_togglebutton    = 1;
   rc_config.freqplots_gdir_togglebutton    = 0;
   rc_config.freqplots_gviewer_togglebutton = 0;
-  rc_config.freqplots_vswr_togglebutton    = 0;
+  rc_config.freqplots_vswr_togglebutton    = 1;
   rc_config.freqplots_zrlzim_togglebutton  = 0;
   rc_config.freqplots_zmgzph_togglebutton  = 0;
   rc_config.freqplots_smith_togglebutton   = 0;
@@ -417,12 +547,13 @@ Set_Window_Geometry(
     GtkWidget *window,
     gint x, gint y, gint width, gint height )
 {
-  if( (width == 0) || (height == 0) ) return;
-
   /* Set size and position of window */
   /* gtk_widget_hide( window ); this leads to an undecorated window in icewm */
-  gtk_window_resize( GTK_WINDOW(window), width, height );
-  gtk_window_move( GTK_WINDOW(window), x, y );
+  if (width && height)
+    gtk_window_resize( GTK_WINDOW(window), width, height );
+
+  if (x >= 0 && y >= 0)
+    gtk_window_move( GTK_WINDOW(window), x, y );
 
 } /* Set_Window_Geometry() */
 
@@ -438,16 +569,15 @@ Restore_Windows( gpointer dat )
   GtkWidget *widget;
 
   /* Open frequency plots window if state data available */
-  if( rc_config.freqplots_width &&
-      rc_config.freqplots_height )
+  if( rc_config.main_loop_start || isFlagSet(OPTIMIZER_OUTPUT) ||
+	  (rc_config.freqplots_is_open && rc_config.freqplots_width && rc_config.freqplots_height))
   {
     widget = Builder_Get_Object( main_window_builder, "main_freqplots" );
     gtk_menu_item_activate( GTK_MENU_ITEM(widget) );
   }
 
   /* Open radiation pattern window if state data available */
-  if( rc_config.rdpattern_width &&
-      rc_config.rdpattern_height )
+  if( rc_config.rdpattern_is_open && rc_config.freqplots_width && rc_config.freqplots_height)
   {
     widget = Builder_Get_Object( main_window_builder, "main_rdpattern" );
     gtk_menu_item_activate( GTK_MENU_ITEM(widget) );
@@ -544,19 +674,48 @@ Restore_GUI_State( void )
 Read_Config( void )
 {
   char
+    home[PATH_MAX],
     fpath[FILENAME_LEN], /* File path to xnec2crc */
     line[LINE_LEN];
   int lnum;
 
+  struct stat st;
+  int umsk, newmode;
+
   /* Config and mnemonics file pointer */
   FILE *fp = NULL;
 
+
+  /* Create the dir if missing */
+  snprintf( fpath, sizeof(fpath), "%s/.xnec2c", get_conf_dir(home, sizeof(home)));
+  if( access(fpath, R_OK) < 0 && errno == ENOENT)
+	  mkdir(fpath, 0755);
+
+  // In commit 4e62893b the mkdir call used 755 instead of 0755 so
+  // permissions were broken.  This fixes that:
+  if (stat(fpath, &st) != 0)
+	  perror(fpath);
+  st.st_mode &= 01777;
+  umask(umsk = umask(0));
+  if (st.st_mode == 755 || st.st_mode == (755 & (~umsk)))
+  {
+	  newmode = 0755 & (~umsk) & 0777;
+	  pr_notice("Fixed %s directory permissions from 0%o to 0%o from bug introduced in 4e62893b.\n",
+                  fpath, st.st_mode, newmode);
+	  chmod(fpath, newmode);
+  }
+
   /* Setup file path to xnec2c rc file */
-  snprintf( fpath, sizeof(fpath), "%s/%s", getenv("HOME"), CONFIG_FILE );
+  snprintf( fpath, sizeof(fpath), "%s/%s", get_conf_dir(home, sizeof(home)), CONFIG_FILE );
 
   /* Create the file if missing */
   if( access(fpath, R_OK) < 0 && errno == ENOENT)
+  {
+	  rc_config.first_run = 1;
 	  Save_Config();
+  }
+  else
+	  rc_config.first_run = 0;
 
   /* Open xnec2c config file */
   if( !Open_File(&fp, fpath, "r") ) return( FALSE );
@@ -564,6 +723,7 @@ Read_Config( void )
   // Iterate over each line and parse the variables into
   // their references defined by rc_config_vars[].
   lnum = 0;
+
   while ( fgets(line, LINE_LEN, fp) != NULL)
   {
 	  lnum++;
@@ -574,13 +734,13 @@ Read_Config( void )
 	  if (!v)
 	  {
 		  if (line[0] != '#')
-			  printf("%s:%d: Line not parsed: %s\n", fpath, lnum, line);
+			  pr_err("%s:%d: Line not parsed: %s\n", fpath, lnum, line);
 		  continue;
 	  }
 	  
 	  if ( fgets(line, LINE_LEN, fp) == NULL)
 	  {
-		  printf("%s:%d: Early end of file for %s: %s \n", fpath, lnum, v->desc, line);
+		  pr_err("%s:%d: Early end of file for %s: %s \n", fpath, lnum, v->desc, line);
 		  break;
 	  }
 
@@ -590,12 +750,36 @@ Read_Config( void )
 	  if (v->ro)
 		  continue;
 
+	  // Skip vars that should be excluded when batch_mode is enabled:
+	  if (rc_config.batch_mode && v->batch_mode_skip)
+		  continue;
+
 	  chomp(line);
 
-	  if (!parse_var(v, line) && line[0] != '#')
-		  printf("%s:%d: parse error (%s): %s \n", fpath, lnum, v->desc, line);
-	  else if (v->init != NULL)
-		  v->init(v, line);
+	  // 1. Skip comments.
+	  // 2. Then try in the "C" locale
+	  // 3. Try in the current locale
+	  // 4. Then try in a known locale with comma for decimals
+	  if (line[0] != '#' &&
+		  !parse_var(v, line, "C") &&
+		  !parse_var(v, line, NULL) &&
+		  !parse_var(v, line, "en_DK")
+		  )
+		  pr_err("%s:%d: parse error (%s): %s \n", fpath, lnum, v->desc, line);
+	  else
+	  {
+		  if (v->builder_check_menu_item_id != NULL && v->builder_window != NULL)
+		  {
+			GtkBuilder *bw = *v->builder_window;
+			GtkWidget *w = Builder_Get_Object(bw, v->builder_check_menu_item_id);
+			int *var = (int*)v->vars[0];
+			gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(w), (gboolean)*var );
+		  }
+
+		  if (v->init != NULL)
+			  v->init(v, line);
+
+	  }
   }
 
   /* Close the config file pointer */
@@ -665,7 +849,7 @@ Get_GUI_State( void )
     gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget) );
 
   /* Get geometry of radiation patterns window */
-  Get_Window_Geometry( rdpattern_window,
+  rc_config.rdpattern_is_open = Get_Window_Geometry( rdpattern_window,
       &(rc_config.rdpattern_x), &(rc_config.rdpattern_y),
       &(rc_config.rdpattern_width), &(rc_config.rdpattern_height) );
 
@@ -710,7 +894,7 @@ Get_GUI_State( void )
   }
 
   /* Get geometry of frequency plots window */
-  Get_Window_Geometry( freqplots_window,
+  rc_config.freqplots_is_open = Get_Window_Geometry( freqplots_window,
       &(rc_config.freqplots_x), &(rc_config.freqplots_y),
       &(rc_config.freqplots_width), &(rc_config.freqplots_height) );
 
@@ -797,6 +981,11 @@ Get_GUI_State( void )
 /* Save_Config()
  *
  * Saves the current values in rc_config to xnec2c.conf
+ *
+ * The format is perhaps unusual.  For historical and backwards-compatibility
+ * reasons the comment indicates the variable on the next line.  It used to be
+ * parsed with a strict ordering, but now rc_config_vars_t defines the output
+ * ordering and assigns values by reference to `rc_config`.
  */
   gboolean
 Save_Config( void )
@@ -804,19 +993,19 @@ Save_Config( void )
   FILE *fp = NULL;  /* File pointer to write config file */
 
   char
+    home[PATH_MAX],
     err_str[300],             /* Error messages string */
     cfg_file[FILENAME_LEN];   /* Path to config file */
 
   /* Setup file path to xnec2c working directory */
-  snprintf( cfg_file, sizeof(cfg_file), "%s/%s", getenv("HOME"), CONFIG_FILE );
+  snprintf( cfg_file, sizeof(cfg_file), "%s/%s", get_conf_dir(home, sizeof(home)), CONFIG_FILE );
 
   /* Open config file for writing */
   if( !Open_File( &fp, cfg_file, "w" ) )
   {
-    snprintf( err_str, sizeof(err_str), "xnec2c: %s", cfg_file );
+    snprintf( err_str, sizeof(err_str), "%s", cfg_file );
     perror( err_str );
-    fprintf( stderr,
-        _("xnec2c: cannot open xnec2c's config file: %s\n"), cfg_file );
+    pr_err("cannot open xnec2c's config file: %s\n", cfg_file);
     return( FALSE );
   }
 
